# coding: utf-8

"""
    Metal API

    # Introduction Equinix Metal provides a RESTful HTTP API which can be reached at <https://api.equinix.com/metal/v1>. This document describes the API and how to use it.  The API allows you to programmatically interact with all of your Equinix Metal resources, including devices, networks, addresses, organizations, projects, and your user account. Every feature of the Equinix Metal web interface is accessible through the API.  The API docs are generated from the Equinix Metal OpenAPI specification and are officially hosted at <https://metal.equinix.com/developers/api>.  # Common Parameters  The Equinix Metal API uses a few methods to minimize network traffic and improve throughput. These parameters are not used in all API calls, but are used often enough to warrant their own section. Look for these parameters in the documentation for the API calls that support them.  ## Pagination  Pagination is used to limit the number of results returned in a single request. The API will return a maximum of 100 results per page. To retrieve additional results, you can use the `page` and `per_page` query parameters.  The `page` parameter is used to specify the page number. The first page is `1`. The `per_page` parameter is used to specify the number of results per page. The maximum number of results differs by resource type.  ## Sorting  Where offered, the API allows you to sort results by a specific field. To sort results use the `sort_by` query parameter with the root level field name as the value. The `sort_direction` parameter is used to specify the sort direction, either either `asc` (ascending) or `desc` (descending).  ## Filtering  Filtering is used to limit the results returned in a single request. The API supports filtering by certain fields in the response. To filter results, you can use the field as a query parameter.  For example, to filter the IP list to only return public IPv4 addresses, you can filter by the `type` field, as in the following request:  ```sh curl -H 'X-Auth-Token: my_authentication_token' \\   https://api.equinix.com/metal/v1/projects/id/ips?type=public_ipv4 ```  Only IP addresses with the `type` field set to `public_ipv4` will be returned.  ## Searching  Searching is used to find matching resources using multiple field comparissons. The API supports searching in resources that define this behavior. Currently the search parameter is only available on devices, ssh_keys, api_keys and memberships endpoints.  To search resources you can use the `search` query parameter.  ## Include and Exclude  For resources that contain references to other resources, sucha as a Device that refers to the Project it resides in, the Equinix Metal API will returns `href` values (API links) to the associated resource.  ```json {   ...   \"project\": {     \"href\": \"/metal/v1/projects/f3f131c8-f302-49ef-8c44-9405022dc6dd\"   } } ```  If you're going need the project details, you can avoid a second API request.  Specify the contained `href` resources and collections that you'd like to have included in the response using the `include` query parameter.  For example:  ```sh curl -H 'X-Auth-Token: my_authentication_token' \\   https://api.equinix.com/metal/v1/user?include=projects ```  The `include` parameter is generally accepted in `GET`, `POST`, `PUT`, and `PATCH` requests where `href` resources are presented.  To have multiple resources include, use a comma-separated list (e.g. `?include=emails,projects,memberships`).  ```sh curl -H 'X-Auth-Token: my_authentication_token' \\   https://api.equinix.com/metal/v1/user?include=emails,projects,memberships ```  You may also include nested associations up to three levels deep using dot notation (`?include=memberships.projects`):  ```sh curl -H 'X-Auth-Token: my_authentication_token' \\   https://api.equinix.com/metal/v1/user?include=memberships.projects ```  To exclude resources, and optimize response delivery, use the `exclude` query parameter. The `exclude` parameter is generally accepted in `GET`, `POST`, `PUT`, and `PATCH` requests for fields with nested object responses. When excluded, these fields will be replaced with an object that contains only an `href` field. 

    The version of the OpenAPI document: 1.0.0
    Contact: support@equinixmetal.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import List, Optional
from pydantic import BaseModel, Field, StrictBool, StrictInt, StrictStr, conlist
from equinix_metal_t0mk.models.bgp_route import BgpRoute

class BgpNeighborData(BaseModel):
    """
    BgpNeighborData
    """
    address_family: Optional[StrictInt] = Field(None, description="Address Family for IP Address. Accepted values are 4 or 6")
    customer_as: Optional[StrictInt] = Field(None, description="The customer's ASN. In a local BGP deployment, this will be an internal ASN used to route within the data center. For a global BGP deployment, this will be the your own ASN, configured when you set up BGP for your project.")
    customer_ip: Optional[StrictStr] = Field(None, description="The device's IP address. For an IPv4 BGP session, this is typically the private bond0 address for the device.")
    href: Optional[StrictStr] = None
    md5_enabled: Optional[StrictBool] = Field(None, description="True if an MD5 password is configured for the project.")
    md5_password: Optional[StrictStr] = Field(None, description="The MD5 password configured for the project, if set.")
    multihop: Optional[StrictBool] = Field(None, description="True when the BGP session should be configured as multihop.")
    peer_as: Optional[StrictInt] = Field(None, description="The Peer ASN to use when configuring BGP on your device.")
    peer_ips: Optional[conlist(StrictStr)] = Field(None, description="A list of one or more IP addresses to use for the Peer IP section of your BGP configuration. For non-multihop sessions, this will typically be a single gateway address for the device. For multihop sessions, it will be a list of IPs.")
    routes_in: Optional[conlist(BgpRoute)] = Field(None, description="A list of project subnets")
    routes_out: Optional[conlist(BgpRoute)] = Field(None, description="A list of outgoing routes. Only populated if the BGP session has default route enabled.")
    __properties = ["address_family", "customer_as", "customer_ip", "href", "md5_enabled", "md5_password", "multihop", "peer_as", "peer_ips", "routes_in", "routes_out"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> BgpNeighborData:
        """Create an instance of BgpNeighborData from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in routes_in (list)
        _items = []
        if self.routes_in:
            for _item in self.routes_in:
                if _item:
                    _items.append(_item.to_dict())
            _dict['routes_in'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in routes_out (list)
        _items = []
        if self.routes_out:
            for _item in self.routes_out:
                if _item:
                    _items.append(_item.to_dict())
            _dict['routes_out'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> BgpNeighborData:
        """Create an instance of BgpNeighborData from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return BgpNeighborData.parse_obj(obj)

        _obj = BgpNeighborData.parse_obj({
            "address_family": obj.get("address_family"),
            "customer_as": obj.get("customer_as"),
            "customer_ip": obj.get("customer_ip"),
            "href": obj.get("href"),
            "md5_enabled": obj.get("md5_enabled"),
            "md5_password": obj.get("md5_password"),
            "multihop": obj.get("multihop"),
            "peer_as": obj.get("peer_as"),
            "peer_ips": obj.get("peer_ips"),
            "routes_in": [BgpRoute.from_dict(_item) for _item in obj.get("routes_in")] if obj.get("routes_in") is not None else None,
            "routes_out": [BgpRoute.from_dict(_item) for _item in obj.get("routes_out")] if obj.get("routes_out") is not None else None
        })
        return _obj


